
// openpluginlib - A plugin interface to openlibraries.

// Copyright (C) 2005-2006 VM Inc.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#include <cassert>
#include <iostream>

#include <openpluginlib/pl/pcos/property.hpp>
#include <openpluginlib/pl/pcos/observer.hpp>
#include <openpluginlib/pl/string.hpp>

namespace opl = olib::openpluginlib;
namespace pcos = olib::openpluginlib::pcos;

class counting_observer : public pcos::observer
{
public:
    counting_observer()
        : count( 0 )
        {}

    virtual void updated( pcos::subject* )
    {
        ++count;
    }

    unsigned int count;
};

int main( )
{
    // check the key value
    {
        pcos::property p1( pcos::key::fromString( "foobar" ) );
        assert( p1.getKey() == pcos::key::fromString( "foobar" ) );
    }

    {
        // assignment from primitive
        pcos::property p1( pcos::key::fromString( "string-key" ) );
        p1 = opl::wstring( L"hello, world" );

        pcos::property p2( pcos::key::fromString( "double-key" ) );
        p2 = 3.141;

        pcos::property p3( pcos::key::fromString( "int-key" ) );
        p3 = 3;

        // assignment to another property
        pcos::property p1dash( p1 );
        pcos::property p2dash( p2 );
        pcos::property p3dash( p3 );

        // check types
        assert( p1.isA< opl::wstring >() );
        assert( !p1.isA< double >() );
        assert( !p1.isA< int >() );

        assert( !p2.isA< opl::wstring >() );
        assert( p2.isA< double >() );
        assert( !p2.isA< int >() );

        assert( !p3.isA< opl::wstring >() );
        assert( !p3.isA< double >() );
        assert( p3.isA< int >() );
            
        // check values
        assert( p1.value< opl::wstring >() == opl::wstring( L"hello, world" ) );
        assert( p2.value< double >() == 3.141 );
        assert( p3.value< int >() == 3 );
    }

    // subject/observer
    {
        boost::shared_ptr< counting_observer > obs( new counting_observer );
        pcos::property p( pcos::key::fromString( "string-key" ) );
        p.attach( obs );

        p = opl::wstring( L"hello, world" );
        assert( obs->count == 1 );

        pcos::property p2( pcos::key::fromString( "string2-key" ) );
        p2 = opl::wstring( L"goodbye, cruel world" );
        p = p2;
        assert( obs->count == 2 );
    }

    return 0;
}
